@@ -1,5 +1,4 @@
 import os
-from datetime import datetime
 from fabric.api import *
 from fabric.utils import _AttributeDict as fdict
 from fabric.contrib.files import contains, exists, append, sed
@@ -31,6 +30,7 @@ def setup_env():
         env.fg.workspace = os.path.join(env.fg.home, 'dev')
         env.fg.inasafe_git_url = 'git://github.com/AIFDR/inasafe.git'
         env.fg.qgis_git_url = 'git://github.com/qgis/Quantum-GIS.git'
+        env.fg.kandan_git_url = 'git://github.com/kandanapp/kandan.git'
         env.fg.inasafe_checkout_alias = 'inasafe-fabric'
         env.fg.qgis_checkout_alias = 'qgis-fabric'
         env.fg.inasafe_code_path = os.path.join(
@@ -74,7 +74,39 @@ def add_ubuntugis_ppa():
     """Ensure we have ubuntu-gis repos."""
     fabtools.deb.update_index(quiet=True)
     fabtools.require.deb.ppa(
-        'ppa:ubuntugis/ubuntugis-unstable', auto_yes=True)
+        #'ppa:ubuntugis/ubuntugis-unstable', auto_yes=True)
+        'ppa:ubuntugis/ubuntugis-unstable')
+
+
+@task
+def setup_latex():
+    """Install latex and friends needed to generate sphinx pdfs."""
+    fabtools.deb.update_index(quiet=True)
+    fabtools.require.deb.package('texlive-latex-extra')
+    fabtools.require.deb.package('texinfo')
+    fabtools.require.deb.package('texlive-fonts-recommended')
+
+
+@task
+def setup_sphinx():
+    """Install sphinx from pip.
+
+    We prefer from pip as ubuntu packages are usually old."""
+    sudo('pip install sphinx')
+
+
+@task
+def setup_transifex():
+    """Install transifex client."""
+    sudo('pip install transifex-client')
+
+
+@task
+def setup_devtools():
+    """Install various useful tools needed for developers."""
+    fabtools.require.deb.package('qtcreator')
+    fabtools.require.deb.package('qt4-designer')
+    fabtools.require.deb.package('qt4-linguist-tools')
 
 
 @task
@@ -91,6 +123,7 @@ def clone_qgis(branch='master'):
     # Add this to the users git config so that we don't get repeated
     # authentication requests when using ssl
     run('git config --global credential.helper \'cache --timeout=3600\'')
+    run('git config --global push.default simple')
 
     code_base = '%s/cpp' % env.fg.workspace
     code_path = '%s/Quantum-GIS' % code_base
@@ -123,26 +156,65 @@ def clone_qgis(branch='master'):
 
 
 @task
-def install_qgis1_8():
-    """Install QGIS 1.8 under /usr/local/qgis-1.8."""
-    setup_env()
-    add_ubuntugis_ppa()
-    sudo('apt-get build-dep -y qgis')
+def setup_ccache():
+    """Setup ccache."""
+    fabtools.require.deb.package('ccache')
+    sudo('ln -fs /usr/bin/ccache /usr/local/bin/gcc')
+    sudo('ln -fs /usr/bin/ccache /usr/local/bin/g++')
+    sudo('ln -fs /usr/bin/ccache /usr/local/bin/cc')
+
+
+@task
+def setup_inasafe():
+    """Setup requirements for InaSAFE."""
+    fabtools.require.deb.package('pep8')
+    fabtools.require.deb.package('pylint')
+    fabtools.require.deb.package('python-nose')
+    fabtools.require.deb.package('python-nosexcover')
+
+
+def compile_qgis(build_path, build_prefix):
+    fabtools.require.deb.package('python-gdal')
     fabtools.require.deb.package('cmake-curses-gui')
     fabtools.require.deb.package('git')
-    clone_qgis(branch='release-1_8')
-    workspace = '%s/cpp' % env.fg.workspace
-    code_path = '%s/Quantum-GIS' % workspace
-    build_path = '%s/build-qgis18' % code_path
-    build_prefix = '/usr/local/qgis-1.8'
     fabtools.require.directory(build_path)
     with cd(build_path):
         fabtools.require.directory(
             build_prefix,
             use_sudo=True,
             owner=env.fg.user)
-        run('cmake .. -DCMAKE_INSTALL_PREFIX=%s' % build_prefix)
-        run('make install')
+        os_version = run('cat /etc/issue.net')
+        os_version == float(os_version.split(' ')[1])
+        if os_version > 13:
+            extra = '-DPYTHON_LIBRARY=/usr/lib/x86_64-linux-gnu/libpython2.7.so'
+        else:
+            extra = ''
+        cmake = ('cmake .. '
+                 '-DCMAKE_INSTALL_PREFIX=%s '
+                 '-DCMAKE_CXX_COMPILER:FILEPATH=/usr/local/bin/g++ '
+                 '-DQT_QMAKE_EXECUTABLE=/usr/bin/qmake-qt4 '
+                 '-DWITH_MAPSERVER=ON '
+                 '-DWITH_INTERNAL_SPATIALITE=ON '
+                 '%s'
+                 % (build_prefix, extra))
+        run('cmake .. %s' % cmake)
+        processor_count = run('cat /proc/cpuinfo | grep processor | wc -l')
+        run('time make -j %s install' % processor_count)
+
+
+@task
+def install_qgis1_8():
+    """Install QGIS 1.8 under /usr/local/qgis-1.8."""
+    setup_env()
+    add_ubuntugis_ppa()
+    setup_ccache()
+    sudo('apt-get build-dep -y qgis')
+    clone_qgis(branch='release-1_8')
+    workspace = '%s/cpp' % env.fg.workspace
+    code_path = '%s/Quantum-GIS' % workspace
+    build_path = '%s/build-qgis18' % code_path
+    build_prefix = '/usr/local/qgis-1.8'
+    compile_qgis(build_path, build_prefix)
 
 
 @task
@@ -153,37 +225,45 @@ def install_qgis2():
 
     """
     setup_env()
+    setup_ccache()
     add_ubuntugis_ppa()
     sudo('apt-get build-dep -y qgis')
-    fabtools.require.deb.package('cmake-curses-gui')
-    fabtools.require.deb.package('git')
+
+    fabtools.require.deb.package('python-pyspatialite')
+    fabtools.require.deb.package('python-psycopg2')
+    fabtools.require.deb.package('python-qscintilla2')
+    fabtools.require.deb.package('libqscintilla2-dev')
+    fabtools.require.deb.package('libspatialindex-dev')
+
     clone_qgis(branch='master')
     workspace = '%s/cpp' % env.fg.workspace
     code_path = '%s/Quantum-GIS' % workspace
     build_path = '%s/build-master' % code_path
     build_prefix = '/usr/local/qgis-master'
-    fabtools.require.directory(build_path)
-    with cd(build_path):
-        fabtools.require.directory(
-            build_prefix,
-            use_sudo=True,
-            owner=env.fg.user)
-        run('cmake .. -DCMAKE_INSTALL_PREFIX=%s' % build_prefix)
-
-        run('make install')
+    compile_qgis(build_path, build_prefix)
 
 
 @task
-def require_postgres_user(user, password=''):
+def setup_qgis2_and_postgis():
+    create_postgis_1_5_db('gis', env.user)
+    install_qgis2()
+
+
+@task
+def require_postgres_user(user, password='', createdb=False):
     #sudo('apt-get upgrade')
     # wsgi user needs pg access to the db
     if not fabtools.postgres.user_exists(user):
         fabtools.postgres.create_user(
-            user,
+            name=user,
             password=password,
-            createdb=False,
+            superuser=False,
+            createdb=True,
             createrole=False,
-            connection_limit=20)
+            inherit=True,
+            login=True,
+            connection_limit=None,
+            encrypted_password=False)
 
 
 def setup_postgres_superuser(user, password=''):
@@ -198,7 +278,7 @@ def setup_postgres_superuser(user, password=''):
 
 
 @task
-def setup_postgis():
+def setup_postgis_1_5():
     """Set up postgis.
 
     You can call this multiple times without it actually installing all over
@@ -265,14 +345,14 @@ def create_postgis_1_5_template():
 @task
 def create_postgis_1_5_db(dbname, user):
     """Create a postgis database."""
-    setup_postgis()
-    require_postgres_user(user)
+    setup_postgis_1_5()
     setup_postgres_superuser(env.user)
+    require_postgres_user(user)
     create_user()
     fabtools.require.postgres.database(
         '%s' % dbname, owner='%s' % user, template='template_postgis')
 
-    grant_sql = 'grant all on schema public to %s;' % user
+    grant_sql = 'GRANT ALL ON schema public to %s;' % user
     # assumption is env.repo_alias is also database name
     run('psql %s -c "%s"' % (dbname, grant_sql))
     grant_sql = (
@@ -324,9 +404,96 @@ def create_user():
     fabtools.require.users.sudoer(env.user)
 
 
+@task
+def setup_kandan(branch='master', user='kandan', password='kandan'):
+    """Set up the kandan chat server - see https://github.com/kandanapp/kandan.
+
+    .. note:: I recommend setting up kandan in a vagrant instance."""
+    setup_env()
+    fabtools.require.deb.package('git')
+    code_base = '%s/ruby' % env.fg.workspace
+    code_path = '%s/kandan' % code_base
+    if not exists(code_path):
+        fastprint('Repo checkout does not exist, creating.')
+        run('mkdir -p %s' % code_base)
+        with cd(code_base):
+            run('git clone %s' % env.fg.kandan_git_url)
+    else:
+        fastprint('Repo checkout does exist, updating.')
+        with cd(code_path):
+            # Get any updates first
+            run('git fetch')
+            # Get rid of any local changes
+            run('git reset --hard')
+            # Get back onto master branch
+            run('git checkout master')
+            # Remove any local changes in master
+            run('git reset --hard')
+            # Delete all local branches
+            run('git branch | grep -v \* | xargs git branch -D')
+
+    with cd(code_path):
+        if branch != 'master':
+            run('git branch --track %s origin/%s' % (branch, branch))
+            run('git checkout %s' % branch)
+        else:
+            run('git checkout master')
+        run('git pull')
+
+    fabtools.require.postgres.server()
+    require_postgres_user(
+        user=user,
+        password=password,
+        createdb=True,
+    )
+
+    with cd(code_path):
+        fabtools.require.deb.package('ruby1.9.1-dev')
+        fabtools.require.deb.package('libxslt-dev')
+        fabtools.require.deb.package('libxml2-dev')
+        fabtools.require.deb.package('libpq-dev')
+        fabtools.require.deb.package('libsqlite3-dev')
+        fabtools.require.deb.package('nodejs')
+        # Newer distros
+        #fabtools.require.deb.package('bundler')
+        # ub 12.04
+        fabtools.require.deb.package('ruby-bundler')
+        fabtools.require.deb.package('')
+        sudo('gem install execjs')
+        append('config/database.yml', 'production:')
+        append('config/database.yml', 'adapter: postgresql')
+        append('config/database.yml', 'host: localhost')
+        append('config/database.yml', 'database: kandan_production')
+        append('config/database.yml', 'pool: 5')
+        append('config/database.yml', 'timeout: 5000')
+        append('config/database.yml', 'username: kandan')
+        append('config/database.yml', 'password: %s' % password)
+        run('RUBYLIB=/usr/lib/ruby/1.9.1/rubygems bundle exec rake db:create '
+            'db:migrate kandan:bootstrap')
+
+    fastprint('Kandan server setup is complete. Use ')
+    fastprint('fab <target> fabgis.fabgis.start_kandan')
+    fastprint('to start the server.')
+
+
+@task
+def start_kandan():
+    """Start the kandan server - it will run on port 3000."""
+    setup_env()
+    code_base = '%s/ruby' % env.fg.workspace
+    code_path = '%s/kandan' % code_base
+    if not exists(code_path):
+        setup_kandan()
+    else:
+        with code_path:
+            run('bundle exec thin start')
+
+
 @task
 def ssh_copy_id():
-    """Copy ssh id from local system to remote."""
+    """Copy ssh id from local system to remote.
+    .. note:: Does not work on OSX!
+    """
     command = 'ssh-copy-id %s' % env.host
     local(command)
 
